@@ -6,8 +6,8 @@
 ;; Author: Boris Buliga <boris@d12frosted.io>
 ;; Maintainer: Boris Buliga <boris@d12frosted.io>
 ;; URL: https://github.com/d12frosted/vulpea-ui
-;; Package-Version: 20260118.1232
-;; Package-Revision: 2b07223f4e40
+;; Package-Version: 20260201.944
+;; Package-Revision: 2d06f4ba4f21
 ;; Package-Requires: ((emacs "29.1") (vulpea "2.0.0") (vui "0.1"))
 ;; Keywords: outlines hypermedia
 
@@ -107,11 +107,6 @@ When non-nil, shows a snippet of text around each backlink mention."
   :type 'boolean
   :group 'vulpea-ui)
 
-(defcustom vulpea-ui-backlinks-preview-lines 2
-  "Number of lines to show in backlink previews for prose context."
-  :type 'integer
-  :group 'vulpea-ui)
-
 (defcustom vulpea-ui-backlinks-prose-chars-before 30
   "Number of characters to show before link in prose previews."
   :type 'integer
@@ -314,7 +309,9 @@ Widgets are filtered by predicate and sorted by order."
   "Major mode for the vulpea-ui sidebar buffer.
 \\{vulpea-ui-sidebar-mode-map}"
   :group 'vulpea-ui
-  (setq-local truncate-lines t))
+  (setq-local truncate-lines t)
+  (when (fboundp 'mode-line-invisible-mode)
+    (mode-line-invisible-mode 1)))
 
 
 ;;; Sidebar state (frame-local)
@@ -336,7 +333,7 @@ Used to prevent re-entry during render.")
   "Return the sidebar buffer name for FRAME.
 If FRAME is nil, use the selected frame."
   (let ((frame (or frame (selected-frame))))
-    (format "*vulpea-ui-sidebar:%s*" (frame-parameter frame 'name))))
+    (format "*vulpea-ui-sidebar:%s*" (or (frame-parameter frame 'window-id) ""))))
 
 (defun vulpea-ui--get-sidebar-buffer (&optional frame)
   "Get the sidebar buffer for FRAME, or nil if it doesn't exist."
@@ -500,14 +497,21 @@ Called from `window-buffer-change-functions'."
     (vulpea-ui-sidebar-refresh)))
 
 (defun vulpea-ui--on-idle ()
-  "Handle idle timeout - refresh stats and outline."
+  "Handle idle timeout - soft refresh preserving memos.
+Only widgets whose deps changed (e.g. `buffer-modified-tick' for
+stats and outline) will recompute."
   (when (vulpea-ui--sidebar-visible-p)
     (let* ((frame (selected-frame))
            (main-win (vulpea-ui--get-main-window frame))
            (main-buf (when main-win (window-buffer main-win)))
-           (note (vulpea-ui--get-note-from-buffer main-buf)))
-      (when note
-        (vulpea-ui-sidebar-refresh)))))
+           (note (vulpea-ui--get-note-from-buffer main-buf))
+           (instance (gethash frame vulpea-ui--sidebar-instances)))
+      (when (and note instance
+                 (vui-instance-buffer instance)
+                 (buffer-live-p (vui-instance-buffer instance)))
+        (with-current-buffer (vui-instance-buffer instance)
+          (setq vulpea-ui--current-note note))
+        (vui-update-props instance (list :note note))))))
 
 
 ;;; Utility functions
@@ -588,9 +592,7 @@ For use within widget components."
 (defun vulpea-ui-widget-toggle-at-point ()
   "Toggle the widget collapse state at point."
   (interactive)
-  (let ((widget (widget-at (point))))
-    (when widget
-      (widget-apply widget :action))))
+  (vulpea-ui-follow-link-at-point))
 
 
 ;;; Widget wrapper component
@@ -682,7 +684,10 @@ STRIP-METADATA removes org metadata lines when non-nil."
   :render
   (let ((note (use-vulpea-ui-note)))
     (when note
-      (let* ((stats (vui-use-memo (note)
+      (let* ((note-buf (when (vulpea-note-path note)
+                         (find-buffer-visiting (vulpea-note-path note))))
+             (tick (when note-buf (buffer-modified-tick note-buf)))
+             (stats (vui-use-memo (note tick)
                       (vulpea-ui--compute-stats note)))
              (chars (plist-get stats :chars))
              (words (plist-get stats :words))
@@ -742,8 +747,11 @@ Otherwise reads from disk."
   :render
   (let ((note (use-vulpea-ui-note)))
     (when note
-      (let ((headings (vui-use-memo (note)
-                        (vulpea-ui--parse-headings note))))
+      (let* ((note-buf (when (vulpea-note-path note)
+                         (find-buffer-visiting (vulpea-note-path note))))
+             (tick (when note-buf (buffer-modified-tick note-buf)))
+             (headings (vui-use-memo (note tick)
+                         (vulpea-ui--parse-headings note))))
         (vui-component 'vulpea-ui-widget
           :title "Outline"
           :count (length headings)
@@ -771,9 +779,13 @@ Otherwise reads from disk."
 Returns a list of plists with :title, :level, and :pos."
   (when (and note (vulpea-note-path note))
     (let ((path (vulpea-note-path note))
-          (max-depth vulpea-ui-outline-max-depth))
+          (max-depth vulpea-ui-outline-max-depth)
+          (existing-buf (find-buffer-visiting (vulpea-note-path note))))
       (with-temp-buffer
-        (insert-file-contents path)
+        (if existing-buf
+            (insert (with-current-buffer existing-buf
+                      (buffer-substring-no-properties (point-min) (point-max))))
+          (insert-file-contents path))
         (vulpea-ui--setup-org-mode)
         (let ((headings nil)
               (archive-tag org-archive-tag))
@@ -1260,57 +1272,33 @@ Renders the preview as a clickable button to jump to the mention."
   "Render PREVIEW as a clickable button to jump to PATH at POS."
   (let ((type (plist-get preview :type))
         (on-click (lambda () (vulpea-ui--jump-to-file-position path pos))))
-    (pcase type
-      ('meta
-       (vui-button (concat "⊢ " (plist-get preview :key) ": "
-                           (or (plist-get preview :value) ""))
-         :face 'vulpea-ui-backlink-preview-face
-         :no-decoration t
-         :on-click on-click
-         :help-echo nil))
-      ('header
-       ;; Header context means the link is IN the heading text - make it clickable
-       (vui-button (concat "§ " (plist-get preview :text))
-         :face 'vulpea-ui-backlink-preview-face
-         :no-decoration t
-         :on-click on-click
-         :help-echo nil))
-      ('table
-       (vui-button (concat "▤ " (plist-get preview :text))
-         :face 'vulpea-ui-backlink-preview-face
-         :no-decoration t
-         :on-click on-click
-         :help-echo nil))
-      ('list
-       (vui-button (concat "· " (plist-get preview :text))
-         :face 'vulpea-ui-backlink-preview-face
-         :no-decoration t
-         :on-click on-click
-         :help-echo nil))
-      ('quote
-       (vui-button (concat "> " (plist-get preview :text))
-         :face 'vulpea-ui-backlink-preview-face
-         :no-decoration t
-         :on-click on-click
-         :help-echo nil))
-      ('code
-       (vui-button (concat "λ " (plist-get preview :text))
-         :face 'vulpea-ui-backlink-preview-face
-         :no-decoration t
-         :on-click on-click
-         :help-echo nil))
-      ('footnote
-       (vui-button (concat "† " (plist-get preview :text))
-         :face 'vulpea-ui-backlink-preview-face
-         :no-decoration t
-         :on-click on-click
-         :help-echo nil))
-      (_
-       (vui-button (or (plist-get preview :text) "")
-         :face 'vulpea-ui-backlink-preview-face
-         :no-decoration t
-         :on-click on-click
-         :help-echo nil)))))
+    (let ((indicator (pcase type
+                       ('meta "⊢")
+                       ('header "§")
+                       ('table "▤")
+                       ('list "·")
+                       ('quote ">")
+                       ('code "λ")
+                       ('footnote "†")
+                       (_ nil)))
+          (text (pcase type
+                  ('meta (concat (plist-get preview :key) ": "
+                                 (or (plist-get preview :value) "")))
+                  (_ (or (plist-get preview :text) "")))))
+      (if indicator
+          (vui-hstack
+           :spacing 1
+           (vui-text indicator :face 'vulpea-ui-backlink-context-face)
+           (vui-button text
+             :face 'vulpea-ui-backlink-preview-face
+             :no-decoration t
+             :on-click on-click
+             :help-echo nil))
+        (vui-button text
+          :face 'vulpea-ui-backlink-preview-face
+          :no-decoration t
+          :on-click on-click
+          :help-echo nil)))))
 
 (defun vulpea-ui--jump-to-file-position (path pos)
   "Jump to position POS in file at PATH."
@@ -1417,18 +1405,24 @@ Returns a list of plists with :note and :count, sorted by title."
          (buf-name (vulpea-ui--sidebar-buffer-name frame))
          (buf (get-buffer-create buf-name))
          (sidebar-win (vulpea-ui--get-sidebar-window frame))
-         (original-window (selected-window)))
+         (original-window (selected-window))
+         (existing-instance (gethash frame vulpea-ui--sidebar-instances)))
     ;; Select sidebar window before mount (vui-mount calls switch-to-buffer)
     (when sidebar-win
       (select-window sidebar-win t))
     (with-current-buffer buf
-      ;; Mount fresh - vui-mount calls kill-all-local-variables
-      (let ((new-instance
-             (vui-mount
-              (vui-component 'vulpea-ui-sidebar-root :note note)
-              buf-name)))
-        (puthash frame new-instance vulpea-ui--sidebar-instances))
-      ;; Set current note AFTER mount (vui-mount kills local variables)
+      (if (and existing-instance
+               (vui-instance-buffer existing-instance)
+               (buffer-live-p (vui-instance-buffer existing-instance)))
+          ;; Reuse existing instance - update props, preserve memos
+          (vui-update-props existing-instance (list :note note))
+        ;; Mount fresh - first render or instance was lost
+        (let ((new-instance
+               (vui-mount
+                (vui-component 'vulpea-ui-sidebar-root :note note)
+                buf-name)))
+          (puthash frame new-instance vulpea-ui--sidebar-instances)))
+      ;; Set current note AFTER render (vui-mount kills local variables)
       (setq vulpea-ui--current-note note)
       (goto-char (point-min)))
     ;; Restore original window
@@ -1488,14 +1482,19 @@ Returns a list of plists with :note and :count, sorted by title."
 
 ;;;###autoload
 (defun vulpea-ui-sidebar-refresh ()
-  "Force refresh the sidebar content."
+  "Force refresh the sidebar, invalidating all caches."
   (interactive)
   (let* ((frame (selected-frame))
          (main-win (vulpea-ui--get-main-window frame))
          (main-buf (when main-win (window-buffer main-win)))
-         (note (vulpea-ui--get-note-from-buffer main-buf)))
-    (setq vulpea-ui--current-note nil)  ; Force update
-    (vulpea-ui--render-sidebar note frame)))
+         (note (vulpea-ui--get-note-from-buffer main-buf))
+         (instance (gethash frame vulpea-ui--sidebar-instances)))
+    (when (and note instance
+               (vui-instance-buffer instance)
+               (buffer-live-p (vui-instance-buffer instance)))
+      (with-current-buffer (vui-instance-buffer instance)
+        (setq vulpea-ui--current-note note))
+      (vui-update instance (list :note note)))))
 
 
 ;;; Built-in widget registration
